# Generated by Django 3.1.1 on 2020-09-23 06:58

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=128, null=True)),
                ('email', models.EmailField(max_length=254)),
                ('phone', models.CharField(blank=True, max_length=128, null=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('avatar', models.URLField(blank=True, null=True)),
                ('provider', models.CharField(choices=[('default', 'default'), ('facebook', 'facebook'), ('google', 'google'), ('apple', 'apple')], default='default', max_length=128)),
                ('social_id', models.CharField(blank=True, max_length=256, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='profile', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.contrib.auth.models import User
from revauth import exceptions
from rest_framework import response
from revauth.settings import api_settings
from revauth.jwt import JWTCryptor
from revauth.serializers import ProfileJWTSerializer
import requests
from django.conf import settings


class BaseHandler:
    jwt_serializer_class = ProfileJWTSerializer
    profile_class = api_settings.DEFAULT_PROFILE_CLASS
    identity_type = api_settings.DEFAULT_IDENTITY
    issuer = api_settings.DEFAULT_VALIDATION_ISSUER
    DEBUG = settings.DEBUG
    
    @property
    def auth_host(self):
        if self.DEBUG:
            return f'https://auth-stg.revtel-api.com/v2/validation/request?issuer={self.issuer}&client_id={settings.CLIENT_ID}'
        return f'https://auth.revtel-api.com/v2/validation/request?issuer={self.issuer}&client_id={settings.CLIENT_ID}'

    def send_validation(self, identity):
        resp = requests.post(self.auth_host, json={'identity': identity})
        resp_json = resp.json()
        resp.raise_for_status()
        return resp_json
        
    def validate(self, identity): 
        user = User.objects.filter(username=identity)
        if user:
            raise exceptions.UserExistsError
        return self.send_validation(identity)

    def on_register(self, **kwargs):
        raise NotImplementedError('on_register')

    def on_login_success(self, profile):
        raise NotImplementedError('on_profile_create')

    def perform_register(self, **kwargs):
        profile = self.on_register(**kwargs)
        resp = self.on_login_success(profile)
        return response.Response(resp, 200)



class DefaultHandler(BaseHandler):
    def on_register(self, decoded, password):
        try:
            identity = decoded['idy']
            user = User.objects.create_user(username=identity, password=password)
        except User.UserExists:
            raise exceptions.UserExistsError
        except KeyError:
            raise exceptions.JWTDecodeError

        data = {self.identity_type: identity}
        try:
            profile = user.profile
        except:
            profile = self.profile_class.objects.create(user=user)

        for key, value in data.items():
            setattr(profile, key, value)

        profile.save()

        return profile

    def on_login_success(self, profile):
        serialized = self.jwt_serializer_class(profile).data
        tokens = JWTCryptor().encode(serialized)
        return {**tokens, **serialized}



from revns.django import EmailNotification as EmailNtfn, SmsNotification as SmsNtfn


class CodeEmailNtfn(EmailNtfn):
    def build_target(self):
        return self.instance.identity

    def build_title(self):
        return 'REVAUTH VALIDATION'

    def build_body(self):
        code = self.instance.code
        return code


class CodeSmsNtfn(SmsNtfn):
    def build_target(self):
        return self.instance.identity

    def build_title(self):
        return 'REVAUTH VALIDATION'

    def build_body(self):
        return self.instance.code



from rest_framework import views, response, permissions, generics
from revauth import exceptions
from django.contrib.auth.models import User
from django.utils import timezone
from datetime import timedelta
from revauth.settings import api_settings
from .serializers import RegisterSerializer, SocialSerializer, ValidationSerializer
from revauth import exceptions
from revauth.jwt import JWTCryptor
from revauth.socials import SocialSDK
from rest_framework import views
from django.conf import settings
from urllib.parse import urlparse


class ValidationView(views.APIView):
    permission_classes = ()
    serializer_class = ValidationSerializer
    handler_class = api_settings.DEFAULT_HANDLER_CLASS

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        identity = serializer.validated_data['identity']
        handler = self.handler_class()
        result = handler.validate(identity)
        return response.Response(result, 200)


class UserRegister(views.APIView):
    profile_class = api_settings.DEFAULT_PROFILE_CLASS
    permission_classes = ()
    serializer_class = api_settings.DEFAULT_REGISTER_SERIALIZER
    handler_class = api_settings.DEFAULT_HANDLER_CLASS

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        if not serializer.is_valid(raise_exception=True):
            return response.Response({'error': 'not valid'})

        try:
            decoded = JWTCryptor().decode(token)
        except:
            raise exceptions.JWTDecodeError

        handler = self.handler_class()
        return handler.perform_register(decoded=decoded, **serializer.validated_data)


class BaseSocialView(views.APIView):
    permission_classes = ()
    provider = None
    serializer_class = SocialSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        client = SocialSDK(social_type=self.provider)
        return client.login(serializer.validated_data['token'])


class FacebookLogin(BaseSocialView):
    provider = 'facebook'


class GoogleLogin(BaseSocialView):
    provider = 'google'



from django.conf import settings
from revauth.settings import api_settings
import urllib
import requests
from django.contrib.auth.models import User
from uuid import uuid4
from revauth.jwt import GoogleCryptor
import jwt


class BaseProvider:
    PROFILE_URL = None
    PROFILE_PARAMS = None
    VERSION=api_settings.AUTH_VERSION
    profile_class = api_settings.DEFAULT_PROFILE_CLASS
    handler_class = api_settings.DEFAULT_HANDLER_CLASS
    provider = None

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def get_profile(self, access_token, **kwargs):
        params = self.PROFILE_PARAMS.format(access_token=access_token, **kwargs)
        resp = requests.get(self.PROFILE_URL + params)
        resp_json = resp.json()
        resp.raise_for_status()
        return resp_json

    def map_profile_data(self, data):
        raise NotImplementedError('map profile data')

    def process_login(self, name, social_id, email='', phone=''):
        profile = self.profile_class.objects.filter(provider=self.provider, social_id=social_id)
        if not profile:
            user = User.objects.create_user(username=uuid4().__str__())
            try:
                profile = user.profile
            except:
                profile = self.profile_class.objects.create(user=user, email=email, phone=phone, name=name, provider=self.provider, social_id=social_id)

        return profile
    
    def login(self, token, **kwargs):
        profile_data = self.get_profile(token)
        mapped = self.map_profile_data(profile_data)
        profile = self.process_login(**mapped)
        handler = self.handler_class()
        result = handler.on_login_success(profile)
        return result

    def get_avatar(self, **data):
        raise NotImplementedError


class FacebookProvider(BaseProvider):
    PROFILE_URL = 'https://graph.facebook.com/v8.0/me'
    PROFILE_PARAMS = '?access_token={access_token}&fields=id,name,email'

    def map_profile_data(self, data):
        return {
            'social_id': data['id'],
            **data
        }


class GoogleProvider(BaseProvider):
    PROFILE_URL = 'https://people.googleapis.com/v1/people/me'
    PROFILE_PARAMS = '?personFields=emailAddresses,names&access_token={access_token}'
    cryptor_class = GoogleCryptor
    provider = 'google'

    def get_profile(self, access_token):
        headers = jwt.get_unverified_header(access_token)
        cryptor = self.cryptor_class(headers)
        return cryptor.decode(access_token)

    def map_profile_data(self, data):
        return {
            'social_id': data['sub'],
            'email': data['email'],
            'name': data['name']
        }



from django.conf import settings
from django.http import HttpResponse
from api.models.redirect_state import RedirectState
from django.shortcuts import get_object_or_404, redirect
from authorization.models.user_profile import UserProfile
from django.contrib.auth.models import User
import json
from ..utils.getUniIdentifier import getUniIdentifier
from ..utils.url import add_url_query
from django.conf import settings


class SignInSDK:
    social_type = None

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def get_provider_class(self):
        from .providers import FacebookProvider, GoogleProvider
        if self.social_type == 'facebook':
            return FacebookProvider
        elif self.social_type == 'google':
            return GoogleProvider

    def get_provider(self):
        provider_class = self.get_provider_class()
        return provider_class(API_KEY=settings.API_KEY)

    def get_callback_url(self, state):
        return '{}/auth/{}/callback?state={}'.format(settings.API_HOST, self.social_type, state)

    def request(self, req_redirect_url, is_mobile=False):
        provider = self.get_provider()
        state = RedirectState.objects.create(
            redirect_url=req_redirect_url,
            is_mobile=is_mobile
        )
        callback_url = self.get_callback_url(state=state.state)
        url = provider.request(state=state, callback_url=callback_url)
        return redirect(url)

    def callback(self, request):
        provider = self.get_provider()
        state = request.query_params.get('state')
        state = get_object_or_404(RedirectState, state=state)
        access_token = request.query_params.get('token')
        error = request.query_params.get('error')
        if error:
            return self.error_resp()

        meta = provider.get_profile(access_token)
        meta['social_type'] = meta.get('provider')

        token = self.on_login_success(
            social_type=self.social_type,
            social_id=meta.get('social_id'),
            meta=meta
        )

        if state.is_mobile:
            return self.static_result_view(token)

        result_url = self.get_result_url(state, token)
        return redirect(result_url)

    def on_login_success(self, social_id, meta: dict):
        profile = UserProfile.objects.filter(social_id=social_id,
                                             social_type=self.social_type).first()
        if not profile:
            user = User.objects.create_user(username=getUniIdentifier(User.objects, field_name='username'))
            profile = user.profile
        for k, v in meta.items():
            if k == 'id':
                continue
            if hasattr(profile, k):
                setattr(profile, k, v)
        profile.save()
        return profile.user.auth_token.key

    def error_resp(self, is_mobile=False):
        if is_mobile:
            return self.static_failure_view()
        return redirect(settings.WEB_HOST + '?error=failed')

    def random_string_digits(self, string_length=12):
        import string
        import random
        """Generate a random string of letters and digits """
        letters_and_digits = string.ascii_letters + string.digits
        return ''.join(random.choice(letters_and_digits) for i in range(string_length))

    def static_result_view(self, token):
        html = '''
            <html>
            <head>
            </head>
            <body style="display: flex; flex-direction: column; align-items: center;">
              <img src="{}/static/telu-logo.png" width="250px" style="display: block; margin: 60px auto 10px;"/>
                <h1 style="text-align: center; color: #0069AB; font-size: 50px;">登入成功</h1>
            <img src="{}/static/telu-icon.png" width="250px" style="display: block; margin: 60px auto 10px;"/>
            <p style="font-size: 40px;text-align: center; color: #0069AB; letter-spacing: 20px;margin-bottom: 300px;">在此開啟您的全球漫遊服務</p>

              <a href="{}://social-signin?token={}"
                style="font-size: 40px; width: 200px; border-radius: 40px;color: #ffffff; background-color:#0069AB; padding: 20px 80px; text-decoration: none; text-align: center;" >
                開啟App
              </a>
            </body>
            </html>



            '''.format(settings.API_HOST,
                       settings.API_HOST,
                       settings.IOS_BUNDLE_ID, token)
        return HttpResponse(html)

    def static_failure_view(self, **kwargs):
        html = '''
                <html>
                <head>
                </head>
                <body style="display: flex; flex-direction: column; align-items: center;">
                  <img src="{}/static/telu-logo.png" width="250px" style="display: block; margin: 60px auto 10px;"/>
                    <h1 style="text-align: center; color: red; font-size: 50px;">登入失敗</h1>
                <img src="{}/static/telu-icon.png" width="250px" style="display: block; margin: 60px auto 10px;"/>
                <p style="font-size: 40px;text-align: center; color: #0069AB; letter-spacing: 20px;margin-bottom: 300px;">在此開啟您的全球漫遊服務</p>
                <a href="{}://social-signin"
                style="font-size: 40px; width: 200px; border-radius: 40px;color: #ffffff; background-color:#0069AB; padding: 20px 80px; text-decoration: none; text-align: center;" >
                開啟App
              </a>
                </body>
                </html>
            '''.format(settings.API_HOST,
                       settings.API_HOST,
                       settings.IOS_BUNDLE_ID)
        return HttpResponse(html)

    def get_result_url(self, state, token, **kwargs):
        result_url = add_url_query(url=state.redirect_url,
                                   token=token
                                   )
        return result_url



from rest_framework import serializers
from revauth.settings import api_settings


class ValidationSerializer(serializers.Serializer):
    identity = serializers.CharField()


class RegisterSerializer(serializers.Serializer):
    password = serializers.CharField()
    access_token = serializers.CharField()


class ProfileJWTSerializer(serializers.Serializer):
    groups = serializers.SerializerMethodField()
    id = serializers.CharField()

    def get_groups(self, obj):
        if obj.is_staff:
            return ['admin']
        return []

    class Meta:
        model = api_settings.DEFAULT_PROFILE_CLASS
        fields = serializers.ALL_FIELDS


class SocialSerializer(serializers.Serializer):
    token = serializers.CharField()
